#include "Wire.h"
#include "LiquidCrystal_I2C.h"
#include "OneWire.h"
#include "DS3231.h"
#include "Adafruit_Sensor.h"
#include "Adafruit_BME280.h"
#include "BH1750.h"

LiquidCrystal_I2C lcd(0x27, 20, 4);
DS3231 clock_1;
RTCDateTime dt;
Adafruit_BME280 bme;
BH1750 lightMeter;

const uint8_t pin_temperature_sensors = 14; //D5
const uint8_t pin_relay_light = 13; // D7
const uint8_t pin_relay_aeration = 12; // D6
const uint8_t pin_button_green = 0; // D3
const uint8_t pin_button_red = 2; // D4

bool           button_click_event = true; // Подія натискання на кнопку
bool           button_click_event_2 = true; // Подія натискання на кнопку
bool           button_state_1 = false; // Стан кнопки
bool           button_state_2 = false; // Стан кнопки
uint8_t        menu_page_number_value = 0; // Значення номеру сторінки меню
uint8_t        mode_light = 0;
uint8_t        mode_aeration = 0;
uint32_t       backlight_working_time_value = 0; // Значення часу роботи підсвічування дисплея
uint32_t       menu_page_refresh_time_value = 0; // Значення часу оновлення сторінки меню

uint32_t       time_work_air = 0;
bool           flag_work_air = 0;
bool           relay_air_state = 0;
bool           relay_laght_state = 0;

float           temerature_air = 0;
float           humidity_air = 0;
float           temerature_solution = 0;
float           lux = 0;

uint8_t         address_sensor[8] = {0x28, 0x60, 0xF4, 0x57, 0x04, 0xE1, 0x3D, 0x56};

OneWire DS18B20(pin_temperature_sensors);

void setup()
{
  Serial.begin(115200);
  Wire.begin();
  lcd.begin();
  clock_1.begin();
  bme.begin();
  lightMeter.begin();

  pinMode(pin_temperature_sensors, INPUT_PULLUP);
  pinMode(pin_button_green, INPUT_PULLUP);
  pinMode(pin_button_red, INPUT_PULLUP);
  pinMode(pin_relay_light, OUTPUT);
  pinMode(pin_relay_aeration, OUTPUT);

  // clock_1.setDateTime(__DATE__, __TIME__);
  // Manual (YYYY, MM, DD, HH, II, SS
  // clock_1.setDateTime(2024, 2, 18, 0, 10, 00);

  lcd.backlight();
lcd.setCursor(4, 1);
lcd.print("HYDROGARDEN");
delay(3000);

}

void loop()
{
  dt = clock_1.getDateTime();

  if (millis() >= (backlight_working_time_value + 30000)) // Умова, при виконанні якої вимикається підсвічування екрана
  {
    lcd.noBacklight(); // Вимикання підсвічування відбувається через 15 секунд після останньої взаємодії з системою
  }
  //------------------------------------------------------------------------------------
  button_state_1 = digitalRead(pin_button_green);
  button_state_2 = digitalRead(pin_button_red);
  //------------------------------------------------------------------------------------
  if (!button_state_2 and menu_page_number_value == 0)
  {
    backlight_working_time_value = millis();
    lcd.backlight();
    delay(200);
  }

  if (!button_state_1 and !button_click_event) // Умова працює таким чином, що при натисканні кнопки умова спрацьовує один раз, повторно може спрацювати лише післі відпускання та повторного натискання кнопки
  {
    button_click_event = true;
    backlight_working_time_value = millis();
    lcd.backlight();
    delay(200);
    menu_page_number_value++; // Інкримент
    if (menu_page_number_value > 2) // Якщо сторінка меню більше за 2 - скидаємо її до 0
    {
      menu_page_number_value = 0;
    }
  }
  else if (button_state_1)
  {
    button_click_event = false;
  }
  //------------------------------------------------------------------------------------
  if (!button_state_2 and !button_click_event_2 and menu_page_number_value == 1)
  {
    button_click_event_2 = true;
    backlight_working_time_value = millis();
    lcd.backlight();
    delay(200);
    mode_light++; // Інкримент
    if (mode_light > 2)
    {
      mode_light = 0;
    }
  }
  else if (button_state_2)
  {
    button_click_event_2 = false;
  }
  //------------------------------------------------------------------------------------
  if (!button_state_2 and !button_click_event_2 and menu_page_number_value == 2)
  {
    button_click_event_2 = true;
    backlight_working_time_value = millis();
    lcd.backlight();
    delay(200);
    mode_aeration++; // Інкримент
    if (mode_aeration > 2)
    {
      mode_aeration = 0;
    }
  }
  else if (button_state_2)
  {
    button_click_event_2 = false;
  }
  //------------------------------------------------------------------------------------
  if ((millis() - menu_page_refresh_time_value) > 500) // Час оновлення даних на сторінці меню
  {
    menu_page_refresh_time_value = millis();

    if (menu_page_number_value == 0) // Перша сторінка
    {
      temerature_solution = get_temperature(address_sensor);
      temerature_air = bme.readTemperature();
      humidity_air = bme.readHumidity();
      lux = lightMeter.readLightLevel();

      lcd.clear();

      /*
            lcd.setCursor(12, 0);
            if (dt.hour < 10) // Виведення годин
            {
              lcd.print("0");
            }
            lcd.print(dt.hour);
            lcd.print(":");
            lcd.setCursor(15, 0);

            if (dt.minute < 10) // Виведення хвилин
            {
              lcd.print("0");
            }
            lcd.print(dt.minute);
            lcd.print(":");
            lcd.setCursor(18, 0);

            if (dt.second < 10) // Виведення секунд
            {
              lcd.print("0");
            }
            lcd.print(dt.second);
      */

      lcd.setCursor(0, 0);
      lcd.print("Light: ");
      lcd.setCursor(7, 0);
      lcd.print(lux, 0);
      if (lux < 10)
      {
        lcd.setCursor(9, 0);
      }
      else if (lux < 100)
      {
        lcd.setCursor(10, 0);
      }
      else if (lux < 1000)
      {
        lcd.setCursor(11, 0);
      }
      else if (lux < 10000)
      {
        lcd.setCursor(12, 0);
      }
      else if (lux < 100000)
      {
        lcd.setCursor(13, 0);
      }
      lcd.print("lux");

      lcd.setCursor(0, 1);
      lcd.print("H Air: ");
      lcd.setCursor(7, 1);
      lcd.print(humidity_air, 1);
      lcd.setCursor(12, 1);
      lcd.print("%");

      lcd.setCursor(0, 2);
      lcd.print("T Air: ");
      lcd.setCursor(7, 2);
      lcd.print(temerature_air, 1);
      lcd.setCursor(12, 2);
      lcd.print("*C");

      lcd.setCursor(0, 3);
      lcd.print("T Solution: ");
      lcd.setCursor(12, 3);
      lcd.print(temerature_solution, 1);
      lcd.setCursor(17, 3);
      lcd.print("*C");
    }
    else if (menu_page_number_value == 1) // Друга сторінка
    {
      lcd.clear();

      lcd.setCursor(12, 0);
      if (dt.hour < 10) // Виведення годин
      {
        lcd.print("0");
      }
      lcd.print(dt.hour);
      lcd.print(":");
      lcd.setCursor(15, 0);

      if (dt.minute < 10) // Виведення хвилин
      {
        lcd.print("0");
      }
      lcd.print(dt.minute);
      lcd.print(":");
      lcd.setCursor(18, 0);

      if (dt.second < 10) // Виведення секунд
      {
        lcd.print("0");
      }
      lcd.print(dt.second);

      lcd.setCursor(0, 0);
      lcd.print("LIGHTING");

      lcd.setCursor(0, 2);
      lcd.print("Operating mode: ");

      if (mode_light == 0) // auto
      {
        lcd.setCursor(16, 2);
        lcd.print("auto");
      }
      else if (mode_light == 1) // on
      {
        lcd.setCursor(16, 2);
        lcd.print("on");
      }
      else if (mode_light == 2) // off
      {
        lcd.setCursor(16, 2);
        lcd.print("off");
      }
      //******************************************************
      if (mode_light == 0)
      {
        if (relay_laght_state == 0)
        {
          lcd.setCursor(0, 3);
          lcd.print("Time on: ");
          lcd.setCursor(9, 3);
          if (dt.hour > 20)
          {
            lcd.print((8 + 24) - dt.hour);
            lcd.setCursor(12, 3);
            lcd.print("hour");
          }
          else if (dt.hour >= 0)
          {
            lcd.print(8 - dt.hour);
            lcd.setCursor(11, 3);
            lcd.print("hour");
          }
        }
        else if (relay_laght_state == 1)
        {
          lcd.setCursor(0, 3);
          lcd.print("Time off: ");
          lcd.setCursor(10, 3);
          lcd.print(20 - dt.hour);
          if ((20 - dt.hour) > 9)
          {
            lcd.setCursor(13, 3);
          } else lcd.setCursor(12, 3);
          lcd.print("hour");
        }
      }
      //**************************************************************
    }
    else if (menu_page_number_value == 2) // Третя сторінка
    {
      lcd.clear();

      lcd.setCursor(12, 0);
      if (dt.hour < 10) // Виведення годин
      {
        lcd.print("0");
      }
      lcd.print(dt.hour);
      lcd.print(":");
      lcd.setCursor(15, 0);

      if (dt.minute < 10) // Виведення хвилин
      {
        lcd.print("0");
      }
      lcd.print(dt.minute);
      lcd.print(":");
      lcd.setCursor(18, 0);

      if (dt.second < 10) // Виведення секунд
      {
        lcd.print("0");
      }
      lcd.print(dt.second);

      lcd.setCursor(0, 0);
      lcd.print("AERATION");

      lcd.setCursor(0, 2);
      lcd.print("Operating mode: ");

      if (mode_aeration == 0) // auto
      {
        lcd.setCursor(16, 2);
        lcd.print("auto");
      }
      else if (mode_aeration == 1) // on
      {
        lcd.setCursor(16, 2);
        lcd.print("on");
      }
      else if (mode_aeration == 2) // off
      {
        lcd.setCursor(16, 2);
        lcd.print("off");
      }

      if (mode_aeration == 0)
      {
        if (relay_air_state == 0)
        {
          lcd.setCursor(0, 3);
          lcd.print("Time on: ");
          lcd.setCursor(9, 3);
          lcd.print((((time_work_air + 1200000) - millis()) / 1000 / 60) + 1);
          if (((((time_work_air + 1200000) - millis()) / 1000 / 60) + 1) > 9)
          {
            lcd.setCursor(12, 3);
          } else lcd.setCursor(11, 3);
          lcd.print("min");
        }
        else if (relay_air_state == 1)
        {
          lcd.setCursor(0, 3);
          lcd.print("Time off: ");
          lcd.setCursor(10, 3);
          lcd.print((((time_work_air + 600000) - millis()) / 1000 / 60) + 1);
          if (((((time_work_air + 600000) - millis()) / 1000 / 60) + 1) > 9)
          {
            lcd.setCursor(13, 3);
          } else lcd.setCursor(12, 3);
          lcd.print("min");
        }
      }
    }
  }
  //------------------------------------------------------------------------------------
  if (mode_light == 0) // auto
  {
    if ((dt.hour >= 8) and (dt.hour <= 20))
    {
      digitalWrite(pin_relay_light, HIGH);
      relay_laght_state = 1;
    }
    else
    {
      digitalWrite(pin_relay_light, LOW);
      relay_laght_state = 0;
    }
  }
  else if (mode_light == 1) // on
  {
    digitalWrite(pin_relay_light, HIGH);
  }
  else if (mode_light == 2) // off
  {
    digitalWrite(pin_relay_light, LOW);
  }

  if (mode_aeration == 0) // auto
  {
    //********************************************************
    if (flag_work_air == 0)
    {
      time_work_air = millis();
      flag_work_air = 1;
    }

    if ((millis() - time_work_air) > 600000)
    {
      digitalWrite(pin_relay_aeration, LOW);
      relay_air_state = 0;
      if ((millis() - time_work_air) > 1200000)
      {
        flag_work_air = 0;
      }
    }
    else
    {
      digitalWrite(pin_relay_aeration, HIGH);
      relay_air_state = 1;
    }
    //********************************************************
  }
  else if (mode_aeration == 1) // on
  {
    digitalWrite(pin_relay_aeration, HIGH);
  }
  else if (mode_aeration == 2) // off
  {
    digitalWrite(pin_relay_aeration, LOW);
  }
  //------------------------------------------------------------------------------------
}

float get_temperature(uint8_t address[8])
{
  float temperature_actual = 0;
  int16_t variable_1 = 0;
  uint8_t data_1[12];

  // 0x1F 9  bits 93ms
  // 0x3F 10 bits 175ms
  // 0x5F 11 bits 375ms
  // 0x7F 12 bits 750ms

  DS18B20.reset();
  DS18B20.select(address);
  DS18B20.write(0x4E);
  DS18B20.write(0x4B); // TH
  DS18B20.write(0x46); // TL
  DS18B20.write(0x1F);

  DS18B20.reset();
  DS18B20.select(address);
  DS18B20.write(0x44);
  delay(100);
  DS18B20.reset();
  DS18B20.select(address);
  DS18B20.write(0xBE);
  for (uint8_t i = 0; i < 9; i++)
  {
    data_1[i] = DS18B20.read();
  }
  variable_1 = (data_1[1] << 8) | data_1[0];
  temperature_actual = (float)variable_1 * 0.0625;
  return temperature_actual;
}
